/*
 * Adapted from The MIT License (MIT)
 *
 * Copyright (c) 2018-2022 DaPorkchop_
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
 * is furnished to do so, subject to the following conditions:
 *
 * Any persons and/or organizations using this software must include the above copyright notice and this permission notice,
 * provide sufficient credit to the original authors of the project (IE: DaPorkchop_), as well as provide a link to the original project.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package net.daporkchop.lib.primitive.generator.replacer;

import lombok.NonNull;
import net.daporkchop.lib.common.reference.cache.Cached;
import net.daporkchop.lib.common.util.PorkUtil;
import net.daporkchop.lib.primitive.generator.Context;
import net.daporkchop.lib.primitive.generator.TokenReplacer;
import net.daporkchop.lib.primitive.generator.param.ParameterContext;
import net.daporkchop.lib.primitive.generator.param.primitive.Primitive;
import net.daporkchop.lib.primitive.generator.param.primitive.PrimitiveParameterOptions;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author DaPorkchop_
 */
public class ComplexGenericReplacer implements TokenReplacer {
    public static final Cached<Matcher> COMPLEX_GENERIC_MATCHER = Cached.regex(Pattern.compile("^_G(?:\\d+(?:[A-Z]+)?(?:extends|super)?,?)+_$"));
    public static final Cached<Matcher> COMPLEX_GENERIC_PARAMS = Cached.regex(Pattern.compile("(\\d+)([A-Z]+)?(extends|super)?,?"));

    @Override
    public String replace(@NonNull Context context, @NonNull String text) {
        Matcher matcher = COMPLEX_GENERIC_MATCHER.get().reset(text);
        if (matcher.matches()) {
            List<String> formatted = new ArrayList<>();
            matcher = COMPLEX_GENERIC_PARAMS.get().reset(text);
            while (matcher.find()) {
                ParameterContext<?> param = context.getParams().get(Integer.parseUnsignedInt(matcher.group(1)));
                if (((Primitive) param.value()).isGeneric()) {
                    String name = PorkUtil.fallbackIfNull(matcher.group(2), ((PrimitiveParameterOptions) param.parameter().options()).genericName());
                    String requirement = matcher.group(3);
                    formatted.add((requirement == null ? "" : "? " + requirement + ' ') + name);
                }
            }
            return formatted.isEmpty() ? "" : formatted.stream().collect(Collectors.joining(", ", "<", ">"));
        }
        return null;
    }
}
